/******** <lalVerbatim file="FCTHV"> ********
Author : Wen, L.
$ID$
********* </lalVerbatim> ********/

#ifndef _FCT_H
#define _FCT_H

#include <stdio.h> 
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <math.h>
#include <lal/LALStdlib.h> 
#include <lal/SeqFactories.h>
#include <lal/LALRCSID.h>
#include "LALWrapperInterface.h"
#include <lal/LALStatusMacros.h>
#include <lal/LALFCTInterface.h>
#include "FCTMyPhaseFunc.h" /*  user supplied if #func>4 or if there is a name change */

/*  define this for verbose debugging messages  */
 #undef FCT_SO_DEBUG
/*    #define FCT_SO_DEBUG 1  */
/*  define these for debugging utils in the shared object */
/*    #define FCT_SO_ATTACH_DDD  1 */
#undef FCT_SO_ATTACH_DDD 

#undef FCT_SO_GRAPHING_FUNCS


#ifdef  __cplusplus   /* C++ protection. */
extern "C" {
#endif


NRCSID( FCTGENERALH, "$ID$");

/******** <lalErrTable file="FCTHErrTab"> ********/
#define FCTGENERALH_ENULL 1
#define FCTGENERALH_MSGENULL "Null pointer."
/******** </lalErrTable> ********/


typedef struct
tagFCTEventType
{
  LIGOTimeGPS                      time;
  REAL4                            effDist;
  REAL4                            mass1; 
  REAL4                            mass2;
  REAL4                            eta;
  REAL4                            t0; 
  REAL4                            t2; 
  REAL4                            t3; 
  REAL4                            t4; 
  REAL4                            tC; 
  REAL4                            snr;
  REAL4                            chisq;
  REAL8                            sigma;
  struct tagFCTEventType           *nextEvent;
}FCTEventType;

typedef struct
tagFCTInParamType 
{
  INT4                          dataLength;  /*  initialized in InitSearch()*/
  INT4                          dimension;    
  INT4                          numDataCube;
  INT4                          numEvents;
  LIGOTimeGPS                      epoch;  /* to be assigned in ConditionData */ 
  INT4                           maxF;/* maximum frequency */
  COMPLEX8Vector                *fctInput;  
  LALFCTPlan                    *fctPlan;
  LALFCTDataCube                *cube;
}FCTInParamType;  


/* local structures required  for LAL FCT  Interface initialization */

typedef struct
tagFCTInStrucType 
{
  LALCreateFCTPlanInput  planIn;
  LALFCTSetUnitsInput    setUnitsIn;
  LALFCTPhaseFn          phaseFuncArray; /* Check pointer array to functions, global */
  INT4                   maxFFTSeg;
}FCTInStrucType;    


typedef struct
tagFCTSearchParams /* initialized in InitSearch() */
{
  BOOLEAN                       searchMaster;
  BOOLEAN                       firstTime; /*  need this? */
  UINT4                        *numSlaves;
  INT4                          *startIndices;
  INT4                          *endIndices;
  INT4                          *indicesLength;
  INT4                          *segLengthPerRun;
  INT4                          *numSegPerDim;
  INT4                          totalNumSeg;
  INT4                          segmentCompleted;
  FCTInParamType                *fctInParams;
}FCTSearchParams;

void
LALBuildFCTOutput (
		LALStatus          *status,
		LALSearchOutput    *output,
		INT4                numEvents,
		FCTEventType             *event
    );



void
LALMyFCTInitPlan(LALStatus *status, FCTInStrucType *fctInStrucs, LALFCTPlan  **fctPlan);

void 
LALMyFCTInterf(LALStatus *status, FCTEventType **event, FCTInParamType *fctInParams);

/*
void
LALFCTInitEvent(LALStatus *status, FCTEventType **event);
*/

void
LALFCTBuildEvent(LALStatus *status, COMPLEX8Vector *fctOut, FCTEventType **event,FCTInParamType *fctInParams);


  /* table explanation */

  /*
  IFO	  CHARACTER	2	819		N		
  SEARCH	VARCHAR	24	819		N		
  END_TIME	INTEGER	4	0		N		
  END_TIME_NS	INTEGER	4	0		N		
  IMPULSE_TIME	INTEGER	4	0		Y		
  IMPULSE_TIME_NS INTEGER	4	0	Y		
  AMPLITUDE	REAL	4	0		N		
  EFF_DISTANCE	REAL	4	0		Y		
  COA_PHASE	REAL	4	0		Y		
  MASS1	REAL	4	0		Y		
  MASS2	REAL	4	0		Y		
  MCHIRP	REAL	4	0		Y		
  ETA	REAL	4	0		Y		
  TAU0	REAL	4	0		Y		
  TAU2	REAL	4	0		Y		
  TAU3	REAL	4	0		Y		
  TAU4	REAL	4	0		Y		
  TAU5	REAL	4	0		Y		
  TTOTAL	REAL	4	0		Y		
  SNR	REAL	4	0		Y		
  CHISQ	REAL	4	0		Y		
  CHISQ_DOF	INTEGER	4	0		Y		
  SIGMASQ	DOUBLE	8	0		Y		
  */




#ifdef  __cplusplus
}
#endif  /* C++ protection. */


#endif /* _FCT_H */










