/******** <lalVerbatim file="FCTApplySearchCV"> ******** 
Author: Wen, L. 
$Id: FCTApplySearch.c,v 1.4 2001/05/16 00:00:26 lwen Exp $ 
********* </lalVerbatim> ********/ 

#include <FCTGeneral.h>
#include <FCTApplySearch.h> 
#include <FCTMPI.h>

NRCSID( FCTAPPLYSEARCHC, "$Id: FCTApplySearch.c,v 1.4 2001/05/16 00:00:26 lwen Exp $");

void 
LALApplySearch( 
	       LALStatus *status, 
	       LALSearchOutput *output, 
	       LALSearchInput *input,
	       LALApplySearchParams *applyParams 
	       ) 
{
  INT4  myRank,j;
  FCTSearchParams *params=NULL;
  MPIMessage myMpiMsg; /* initialization message */
  MPI_Comm       myMpiComm; 

  /********************* LAL thing **********************************/ 
  INITSTATUS( status, "LALApplySearch", FCTAPPLYSEARCHC ); 
  ATTATCHSTATUSPTR (status); 
  
  /* check input, output and parameters */
  
  ASSERT (output, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL);
  ASSERT ( !(output->result), status, APPLYSEARCH_ENNUL, APPLYSEARCH_MSGENNUL);

#ifdef FCT_SO_DEBUG
  /*      lalDebugLevel=24; */

#endif
  if ( !(output) ){ 
    ABORT( status, FCTGENERALH_ENULL, FCTGENERALH_MSGENULL);
    
  }

#ifdef FCT_SO_DEBUG  
  printf("in ApplySearch()\n");
#endif
  params=(FCTSearchParams *) applyParams->searchParams;

  if ( !params )
  {
    ABORT (status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC);
  }

  /******************************************************
	     
	  MPI Initialization for all nodes

  ******************************************************/

  if ( !params->numSlaves )
  {
    /* allocate memory and check for success   */

    params->numSlaves = (UINT4 *) LALMalloc( sizeof(UINT4) );
    memset( params->numSlaves, 0, sizeof(UINT4) );
    
    if( !params->numSlaves )
    {
      ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
    }
    
    MPI_Comm_size( *(applyParams->mpiParams->comm), params->numSlaves );
    *(params->numSlaves) -= 1;  /*  do it only once for the first call */
    myMpiMsg.send=1;
  }/* endif firstTime */

  /* get the number of search slaves and my Rank (0 for master) */

  MPI_Comm_rank( *(applyParams->mpiParams->comm), &myRank );
  myMpiComm   = *(applyParams->mpiParams->comm);
  myMpiMsg.source= myRank;
  output->numOutput =0;
  
#ifdef FCT_SO_DEBUG
   debug
   fprintf(stdout,"I am node number %i,",myRank);
   fflush( stdout );
#endif

  /************************************************************************** 
     **************************************************************								       
                          MPI Master activity 
			  
	   (a Naive Master-Slave model, to be modified to be SPMD)
			  
			  Listen to Slaves

			  Calculate fraction work left


     **************************************************************			      
  **************************************************************************/	

  if ( params->searchMaster )
  {
#ifdef FCT_SO_DEBUG
    fprintf(stdout," the ***master***, lazy and always a bottleneck!  \n ");
    fflush( stdout );
#endif

    if ( *(params->numSlaves) )
    {

      /* wait for a message */

#ifdef FCT_SO_DEBUG
      fprintf(stdout,"master of rank %i  waiting for slave's msgs...\n",myRank);
      fflush( stdout );
#endif
      
      LALMPIRecvMsg (status->statusPtr, &myMpiMsg, myMpiComm);
      CHECKSTATUSPTR (status);

      params->segmentCompleted++;

#ifdef FCT_SO_DEBUG
      
      fprintf(stdout,"master of rank %i received the %i th  slave's msgs...\n",	      myRank,params->segmentCompleted);
      fflush( stdout );
#endif

      switch (myMpiMsg.msg)
      {
	
      case RoutineExch: 

	output->fracRemaining = 1.0 - (REAL4) (params->segmentCompleted)
	  /params->totalNumSeg;
	output->notFinished   = 1;

#ifdef FCT_SO_DEBUG
	
	fprintf(stdout,"for node %i, the master,  numSlave, segcom,                 and fracrem= %i,%i,%g\n",myRank,*(params->numSlaves),		params->segmentCompleted,output->fracRemaining ); 
	fflush( stdout );
#endif

	break;

      case ExchFinished:
	--*(params->numSlaves);
	break;
       
	/* unknown type then quit  */
      default: 
	ABORT( status, APPLYSEARCH_EUEXT, APPLYSEARCH_MSGEUEXT );
	
      }/* end switch */
      

    } /* endif params->numSlave !=0 */

    else /*  params->numSlaves=0 */
    {
#ifdef FCT_SO_DEBUG
      
      fprintf(stdout,"master of node %i is out of job ! \n",myRank);
      fflush( stdout );
#endif

      output->fracRemaining = 0;
      output->notFinished   = 0;
    }
  }/*  endif searchMaster */

  
          /***********************************************************
                      **************************************************
								     
                      Slave Activity:		

		      Determine the segment of params to be calculated
		      Call FCT 
		      Talk to Master only 		     
		      Report at completion of each segment of parameters
		      Indicating if there are events or job finished

		      *************************************************	
	 ******************************************************************/ 
	
  
  else  /* if I am a slave */
  {

    INT4                          *startIndices=NULL; 
    INT4                          *endIndices=NULL; 
    INT4                          dimension;
    INT4                          Ntemp;
    INT4                          res;
    FCTEventType                  *event=NULL;


    dimension=params->fctInParams->dimension;
    startIndices =&(params->fctInParams->cube->start_locations[0]);
    endIndices =&(params->fctInParams->cube->end_locations[0]);

#ifdef FCT_SO_DEBUG
    
    fprintf(stdout,"the diligent **slave**, hard working and underpaid \n ");
     fflush( stdout );
#endif

     /*  initialization of the start(end)indices */

     /*  first slave time in applySearch() */
    if (params->firstTime    )
    {  
      params->firstTime=0;
      params->totalNumSeg=params->totalNumSeg/(*(params->numSlaves));


      /* 
	 calculate the starting indices for each slave
	 Check need to be changed if we decide to use master
*/

      
      Ntemp=(myRank-1)*params->totalNumSeg;
      for (j=1; j<dimension; j++)
      {
      	res=Ntemp  % (params->numSegPerDim[j]);
	Ntemp/=params->numSegPerDim[j];
	startIndices[j]=params->startIndices[j]+res*params->segLengthPerRun[j];
	endIndices[j]=startIndices[j]+params->segLengthPerRun[j];

#ifdef FCT_SO_DEBUG
	
	fprintf(stdout," first-time startIndices are %i\n",startIndices[j] );
	fflush( stdout );
	/*  something particular to fct.c */
	fprintf(stdout," first-time endIndices are %i\n",endIndices[j]-1 );
	fflush( stdout );
#endif

      }/* end for */
    }/* end firstTime */

        
      /**************************************************************

		Call FCT 
 
      ***************************************************************/	
    /*  need to make sure that the segnum !=0  */
    
    
    LALMyFCTInterf(status->statusPtr, &event, params->fctInParams);
    CHECKSTATUSPTR (status); 

    if (!event && params->fctInParams->numEvents >0)
    {
      ABORT(status, APPLYSEARCH_ENULL,APPLYSEARCH_MSGENULL);
    }

#ifdef FCT_SO_DEBUG
    
    printf(" numEvents are %i\n",params->fctInParams->numEvents);
#endif

    if (params->fctInParams->numEvents>0)
    {
      

      LALBuildFCTOutput (status->statusPtr, output, params->fctInParams->numEvents, event);
      CHECKSTATUSPTR (status); 
      
      /*  free temporary memory of event */
      while(event)
      {
	FCTEventType *myEvent;
	myEvent=event;
	event=event->nextEvent;
	LALFree(myEvent);

#ifdef FCT_SO_DEBUG
	
    	printf("free event ...\n");
#endif
	
      }/* end ! event */
    }/*  endif numEvent>0 */

    params->segmentCompleted++;

#ifdef FCT_SO_DEBUG
	
    	printf(" done freeing event \n");
#endif

    if (params->segmentCompleted >= params->totalNumSeg)
    {

      output->fracRemaining = 0; 
      output->notFinished = 0; 
      myMpiMsg.msg    =ExchFinished;	
    }
    else
    {

      output->fracRemaining =1-1.0*params->segmentCompleted/params->totalNumSeg;
#ifdef FCT_SO_DEBUG
      
       fprintf(stdout,"for slave %i, numSlave, seg_com, and frac_rem= %i,%i,%g\n",	      myRank,*(params->numSlaves),params->segmentCompleted,	      output->fracRemaining ); 
       fflush( stdout );
#endif


      output->notFinished = 1;
      myMpiMsg.msg    =RoutineExch;

      /*  re-set the startIndices for the next run */
      j=1;
      startIndices[j]+=params->segLengthPerRun[j];
      endIndices[j]=startIndices[j]+params->segLengthPerRun[j];

      while (startIndices[j]>params->endIndices[j])
      {
	startIndices[j]=params->startIndices[j];
	endIndices[j]=startIndices[j]+params->segLengthPerRun[j];
	j++;
	startIndices[j]+= params->segLengthPerRun[j];      
	endIndices[j]=startIndices[j]+params->segLengthPerRun[j];
      }/*  end while  */

#ifdef FCT_SO_DEBUG
      for (j=1;j<dimension;j++)
      {
	fprintf(stdout," next starting start_indices  are %i\n", startIndices[j] );
	fflush( stdout );
	fprintf(stdout," next starting end  _indices  are %i\n", endIndices[j]-1 );
	fflush( stdout );
      }
#endif

    }/* endif finished */
    
    

    LALMPISendMsg (status->statusPtr, &myMpiMsg, 0,myMpiComm);
    CHECKSTATUSPTR (status);    

#ifdef FCT_SO_DEBUG
    
    fprintf(stdout," slave %i sent a msg to master of %i \n",myRank, myMpiMsg.msg );
    fflush( stdout );
#endif

    
  } /* endif slave  */
  

#ifdef FCT_SO_DEBUG
  printf(" out of ApplySearch ()\n");
#endif

  DETATCHSTATUSPTR (status); 
  RETURN( status ); 

} /* end applySearch() */
