/******** <lalVerbatim file="FCTMyInterfCV"> ******** 
Author: Wen, L. 
$Id: FCTMyInterf.c,v 1.4 2001/05/16 00:00:26 lwen Exp $ 
********* </lalVerbatim> ********/ 
#include <FCTGeneral.h>

NRCSID( FCTMYINTERFC, "$Id: FCTMyInterf.c,v 1.4 2001/05/16 00:00:26 lwen Exp $");

/* #define NUM_DATA_CUBES 1  */ /* don't know why it is here */

void
LALMyFCTInitPlan(LALStatus *status, FCTInStrucType *fctInStrucs, LALFCTPlan  **fctPlanPtr)
{
  
  

  LALFCTPlan * fctPlan=0;
  
  INITSTATUS( status, "FCTMYINTERFC", FCTMYINTERFC ); 
  ATTATCHSTATUSPTR (status);
  
  /* equivalent to fct_init_plan() and  fct_add_phase_function() */

   
  LALCreateFCTPlan(status->statusPtr,&fctPlan,&(fctInStrucs->planIn));  
  
  CHECKSTATUSPTR (status);   

  LALFCTSetUnits(status->statusPtr,&(fctInStrucs->setUnitsIn),fctPlan);
  CHECKSTATUSPTR (status);  


  LALFCTSetMaxSegments(status->statusPtr, fctInStrucs->maxFFTSeg, fctPlan);
  CHECKSTATUSPTR (status);   

#ifdef FCT_SO_DEBUG

  if(!fctPlan)
  {
    ABORT( status, 1, "null fctPlan1");
  }
#endif

  *fctPlanPtr=fctPlan;

  
  DETATCHSTATUSPTR (status);
  RETURN( status ); 

}


void 
LALMyFCTInterf(LALStatus *status, FCTEventType **event, FCTInParamType *fctInParams)
{

  
  LALFCTPlan                    *fctPlan;
  UINT8                     outputDataSize=0;
  COMPLEX8Vector*            fctOut=0;
  LALFCTSetDataCubesInput   dataCubesIn;
#ifdef FCT_SO_DEBUG      
  INT4                        j;
#endif  


  INITSTATUS( status, "FCTMYINTERFC", FCTMYINTERFC ); 
  ATTATCHSTATUSPTR (status);

  /*
    fprintf(stdout," in MyFctInterf ()\n");
    fflush( stdout );
  */
   
  fctPlan=fctInParams->fctPlan;


  dataCubesIn.data_cube=fctInParams->cube;
  dataCubesIn.num_data_cubes =fctInParams->numDataCube;

  /* temp debug */
#ifdef FCT_SO_DEBUG      

  printf("before dataCubeIn\n");
  for (j=0;j<fctInParams->dimension;j++)
  {
    printf("cube: start/end locations are %i,%i\n", fctInParams->cube->start_locations[j],fctInParams->cube->end_locations[j]);
  }
#endif
    
  /*  call FCT search here and return event list */

  LALFCTSetDataCubes(status->statusPtr, &dataCubesIn,fctPlan);
  CHECKSTATUSPTR (status);   
  
  LALFCTOutputDataSize(status->statusPtr,&outputDataSize, fctPlan);
  CHECKSTATUSPTR (status);   
   
#ifdef FCT_SO_DEBUG

  for (j=0;j<fctInParams->fctInput->length;j++)
  {
    
    printf("fctInput= %g,%g\n",fctInParams->fctInput->data[j].re, fctInParams->fctInput->data[j].im); 
  }

#endif

  /*  Allocate an output vector of this size  */
  LALCCreateVector(status->statusPtr, &fctOut, outputDataSize);
  CHECKSTATUSPTR (status);

  LALFCTCalculate(status->statusPtr, fctOut, fctInParams->fctInput, fctPlan);
  CHECKSTATUSPTR (status);
#ifdef FCT_SO_DEBUG
  printf("outputDataSize=%u\n", (UINT4) outputDataSize);

  for (j=0;j<outputDataSize;j++)
  {
    printf("fctOut = %g,%g\n",fctOut->data[j].re, fctOut->data[j].im); 
  }
  
#endif

  /*  Check fctInParams->fctInput has to be Complex8  */
  LALFCTBuildEvent(status->statusPtr, fctOut, event, fctInParams); /*  creat a linked eventlist */ 
  CHECKSTATUSPTR (status);

  /*  free allocated memories */

  LALCDestroyVector(status->statusPtr, &fctOut);
  CHECKSTATUSPTR (status);

  
  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
}

/*
void
LALFCTInitEvent(LALStatus *status, FCTEventType **event)
{

  FCTEventType *myEvent;
  myEvent=*event=(FCTEventType *)LALMalloc(sizeof(FCTEventType)) ; //allocate here, deallocate outside
  myEvent->numEvents=0;
  myEvent->fstart=-1;
  myEvent->fend=-1;
  myEvent->tstart=-1;
  myEvent->tend=-1;
  myEvent->whichPlane=-1;
  myEvent->excessPower=0.0;
  myEvent->alpha=0.0;
  myEvent->nextEvent=NULL;
}
*/

void
LALFCTBuildEvent(LALStatus *status, COMPLEX8Vector *fctOut, FCTEventType **event,FCTInParamType *fctInParams)
{
  
  INT4   i,j,dimension,dimLength;
  UINT4  length;
  

  FCTEventType *myEvent;

  REAL8  power=0.;
  REAL8  avgP=0.;
  REAL8  sigmaP=0.;
  REAL8  maxP=0.;
  INT4   maxJJ=0;
  INT4   *maxJ=0;
  INT4 const numJ=5;

  dimension=fctInParams->dimension;
  maxJ=(INT4 *) LALMalloc(sizeof(INT4)*numJ); /*  5 specific to inspiral Check */

  length=fctOut->length;
  for (i=0;i<numJ;i++)
  {
    maxJ[i]=0;
  }

  /*  calculate the average, sigma, and max */

  for (j=0;j<length;j++)
  {
    power=(fctOut->data[j].re) *(fctOut->data[j].re)+(fctOut->data[j].im)*(fctOut->data[j].im);

    if (power>maxP) 
    {
      maxP=power;
      maxJJ=j;
    } /*  endif */
    
    avgP+=power;
    sigmaP+=power*power;
  } /* end for */

  avgP/=length; 
  sigmaP/=(length-1);
  sigmaP -=avgP*avgP/(length-1)*length;
  sigmaP=sqrt(sigmaP);

  /*

   #ifdef FCT_SO_DEBUG
  
     printf(" in LALFCTBuildEvent()\n");
     printf(" length=%i,avgP=%e, sigP=%e, maxP=%e,maxJ=%i,snr=%e and %e\n",length,avgP,sigmaP,maxP,maxJJ,(maxP-avgP)/sigmaP,fctInParams->dataLength/exp(maxP/sigmaP));
     printf("startlocation=%i,%i\n",dimLength, fctInParams->cube->start_locations[1],fctInParams->cube->start_locations[2]);
    
   #endif
  */

  fctInParams->numEvents=0;

  if ( (maxP-avgP) >400*sigmaP) 
       /* 
	  if ( fctInParams->dataLength/exp(maxP/sigmaP) <1.0e-3) 
    if ( (maxP-avgP) >0) 
       */

  {
    
    for (i=0;i<dimension;i++)
    {
      dimLength=(fctInParams->cube->end_locations[i]-fctInParams->cube->start_locations[i ])/fctInParams->cube->stride[i];
      maxJ[i]=fctInParams->cube->start_locations[i ]+(maxJJ % dimLength)*fctInParams->cube->stride[i];
      maxJJ=maxJJ/dimLength;
    }
    /* 
     #ifdef FCT_SO_DEBUG
    
     printf("maxJJ=%i,%i,%i,%i,%i\n",maxJ[0],maxJ[1],maxJ[2],maxJ[3],maxJ[4]);
     #endif
    */

    myEvent= *event=(FCTEventType *)LALMalloc(sizeof(FCTEventType)) ; /* allocate here, deallocate outside */

    fctInParams->numEvents=1;  /*  with  the way we design it here, it is at most 1  */
    myEvent->time.gpsSeconds=fctInParams->epoch.gpsSeconds; 
    myEvent->time.gpsNanoSeconds=fctInParams->epoch.gpsNanoSeconds; 
    myEvent->effDist=0.0;
    myEvent->mass1=1.4;  /* arbitrary number at this time */
    myEvent->mass2=1.4;
    myEvent->eta=(myEvent->mass1)*(myEvent->mass2)/(myEvent->mass1+myEvent->mass2)/(myEvent->mass1+myEvent->mass2);

    myEvent->tC=maxJ[0]; /*  should input the time series start-time  */
    myEvent->t0=maxJ[1]/1228.81374;
    myEvent->t2=maxJ[2]/2405.28188;
    myEvent->t3=maxJ[3];
    myEvent->t4=maxJ[4];
    myEvent->snr=maxP/sigmaP;
    myEvent->chisq=0.9;
    myEvent->sigma=22.36; /*  for inspiral only */
    myEvent->nextEvent=NULL;

  } /*  end if   */
  LALFree(maxJ);


    /* for event analyser with events >1 */

  /*
    myEvent->numEvents=2;   
    numEvents=myEvent->numEvents;
    for (j=0; j<numEvents-1; j++)
    {
      myEvent->fstart=0;
      myEvent->fend=1;
      myEvent->tstart=0;
      myEvent->tend=1;
      myEvent->whichPlane=j;
      myEvent->excessPower=0.5;
      myEvent->alpha=0.9;
      myEvent->nextEvent=(FCTEventType *)LALMalloc(sizeof(FCTEventType));
      myEvent=myEvent->nextEvent;
    }//end for 

    // the one at the end 
    myEvent->fstart=0;
    myEvent->fend=1;
    myEvent->tstart=0;
    myEvent->tend=1;
    myEvent->whichPlane=j;
    myEvent->excessPower=0.5;
    myEvent->alpha=0.9;
    myEvent->nextEvent=NULL;

  */
  
  /*   printf(" out LALFCTBuildEvent()\n"); */
}
