/*----------------------------------------------------------------------- 
 * 
 * File Name: TmpltInput.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: InitSearchTmplt.c,v 1.3 2001/05/17 18:58:48 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "InitSearch.h"

NRCSID( TMPLTINPUTC, PKG "$Id: InitSearchTmplt.c,v 1.3 2001/05/17 18:58:48 duncan Exp $" );


void
InspiralTmpltsFromCmdLine (
    LALStatus                  *status,
    InspiralSearchParams       *params,
    CHAR                      **argv
    )
{
  INT4                  start = 0;
  INT4                  characters = 0;
  INT4                  numTmpltString;
  INT4                  flag;
  UINT4                 tmpltIndex = 0;
  CHAR                 *tmpltString;
  CHAR                 *result; 
  CHAR                  buffer[256];
  REAL4                 m1, m2;
  InspiralTemplate     *here = NULL;

  INITSTATUS( status, "InspiralTmpltsFromCmdLine", TMPLTINPUTC );
  ATTATCHSTATUSPTR( status );


  /*
   *
   * check input and output
   *
   */


  ASSERT( params, status, INITSEARCH_ENULL, INITSEARCH_MSGENULL );
  ASSERT( argv, status, INITSEARCH_ENULL, INITSEARCH_MSGENULL );


  /*
   *
   * set up variables
   *
   */


  numTmpltString = atoi( argv[INSPARG_TPRM] );
  tmpltString = argv[INSPARG_TMPL];


  /*
   *
   * parse the list of templates
   *
   */


  /* find the first semicolon */
  result = strchr( tmpltString, '/' );

  /* if there is no semicolon there should only be one template     */
  /* if there is a semicolon there should be more than one template */
  if ( !result && numTmpltString != 1 ) 
  {
    ABORT( status, INITSEARCH_ETSTR, INITSEARCH_MSGETSTR );
  }
  else if ( result && numTmpltString == 1 ) 
  {
    ABORT( status, INITSEARCH_ETSTR, INITSEARCH_MSGETSTR );
  }

  /* how many chars to copy */
  characters = ( result ) ? 
    result - tmpltString + 1 - start : strlen( tmpltString ) - start + 1;

  /* copt the chars into the buffer */
  strncpy( buffer, tmpltString + start, characters - 1 );
  buffer[characters - 1] = '\0';

  /* increment the start counter */
  start += characters;

  /* parse the line of templates */
  flag = sscanf( buffer, "%f, %f\n", &m1, &m2 ) - 2;

  ASSERT( !flag, status, 
      INITSEARCH_ENDAT, INITSEARCH_MSGENDAT );

  /* allocate the head of the linked list */
  here = params->bankHead = (InspiralTemplate *)
    LALCalloc( 1, sizeof( InspiralTemplate ) );
  ASSERT( here, status, 
      INITSEARCH_EALOC, INITSEARCH_MSGEALOC );

  /* set the template parameters */
  here->number = tmpltIndex++;
  here->mass1 = m1;
  here->mass2 = m2;
  here->totalMass = m1 + m2;
  here->mu = m1*m2/here->totalMass;
  here->eta = here->mu/here->totalMass;

  LALI4CreateVector( status->statusPtr, 
      &(here->segmentIdVec), params->initParams->numSegments );
  CHECKSTATUSPTR( status );
  memset( here->segmentIdVec->data, 0, 
      here->segmentIdVec->length * sizeof(INT4) );

  /* continue with the rest of the string */
  while( start < strlen( tmpltString ) )
  {
    /* find the next semicolon */
    result = strchr( tmpltString + start, '/' );

    /* how many chars to copy */
    characters = ( result ) ? 
      result - tmpltString + 1 - start : strlen( tmpltString ) - start + 1;

    /* copt the chars into the buffer */
    strncpy( buffer, tmpltString + start, characters - 1 );
    buffer[characters - 1] = '\0';

    /* increment the start counter */
    start += characters;

    /* read in the masses and allocate the first template */
    flag = sscanf( buffer, "%f, %f\n", &m1, &m2 ) - 2;
    if ( !flag )
    {
      here->next = (InspiralTemplate *)
        LALCalloc( 1, sizeof( InspiralTemplate ) );
      here = here->next;
      ASSERT( here, status, 
          INITSEARCH_EALOC, INITSEARCH_MSGEALOC );

      here->number = tmpltIndex++;
      here->mass1 = m1;
      here->mass2 = m2;
      here->totalMass = m1 + m2;
      here->mu = m1*m2/here->totalMass;
      here->eta = here->mu/here->totalMass;

      LALI4CreateVector( status->statusPtr, 
          &(here->segmentIdVec), params->initParams->numSegments );
      CHECKSTATUSPTR( status );
      memset( here->segmentIdVec->data, 0, 
          here->segmentIdVec->length * sizeof(INT4) );
    }
    else
    {
      ABORT( status, INITSEARCH_ETSTR, INITSEARCH_MSGETSTR );
    }
  }

  /* record number of templates */
  params->numTmplts = tmpltIndex;

  /* check that we read the correct number of templates */
  if ( tmpltIndex != numTmpltString )
  {
    ABORT( status, INITSEARCH_ETSTR, INITSEARCH_MSGETSTR );
  }      


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
