/********************************** <lalVerbatim file="StochasticBarDataHV">
Author: Anderson, W.G., Heng, I.S., Romano, J.D., and Whelan, J.T.
$Id: StochasticBarData.h,v 1.4 2001/09/02 05:55:57 heng Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticBarDataH">
\section{Header \texttt{StochasticBarData.h}}
\label{s:StochasticBarData.h}

Header file for ...

\subsection*{Synopsis}
\begin{verbatim}
#include "StochasticBarData.h"
\end{verbatim}

\noindent
This header ...

\subsection*{Error conditions}
\input{StochasticBarDataHErrTab}

\subsection*{Structures}

*************************************************** </lalLaTeX> */

#ifndef _STOCHASTICDATA_H
#define _STOCHASTICDATA_H

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/LALRCSID.h>

#ifdef  __cplusplus   /* C++ protection. */
extern "C" {
#endif

NRCSID (STOCHASTICDATAH, "$Id: StochasticBarData.h,v 1.4 2001/09/02 05:55:57 heng Exp $");

/******** <lalErrTable file="StochasticBarDataHErrTab"> ********/
#define STOCHASTICDATAH_ENNUL       1 
#define STOCHASTICDATAH_ENULL       2
#define STOCHASTICDATAH_ENSEG       3
#define STOCHASTICDATAH_ENPTS       4
#define STOCHASTICDATAH_ENFRQ       5
#define STOCHASTICDATAH_EALOC       6
#define STOCHASTICDATAH_ESTID       7

#define STOCHASTICDATAH_MSGENNUL    "Non-null pointer"
#define STOCHASTICDATAH_MSGENULL    "Null pointer"
#define STOCHASTICDATAH_MSGENSEG    "Number of data segments is zero or negative"
#define STOCHASTICDATAH_MSGENPTS    "Number of points in a data segment is zero or negative"
#define STOCHASTICDATAH_MSGENFRQ    "Number of frequencies is zero or negative"
#define STOCHASTICDATAH_MSGEALOC    "Memory allocation error"   
#define STOCHASTICDATAH_MSGESTID    "Detector Site ID out of range"
/******** </lalErrTable> ********/

/******************************* <lalLaTeX file="StochasticBarDataH">

\subsubsection*{\texttt{struct StochasticInitParams}}
\index{\texttt{StochasticInitParams}}

\noindent
The fields are:
\begin{description}
\item[\texttt{UINT4 numSegments}] Number of data segments
\item[\texttt{UINT4 numPoints}] Number of points in a data segment
\item[\texttt{UINT4 numFrequencies}] Number of frequencies in optimal filter
\item[\texttt{REAL8 f0}] Start frequency for optimal filter
\item[\texttt{REAL8 deltaF}] Frequency spacing for optimal filter
\item[\texttt{UINT4 numFrequencies}] Number of frequencies in cross-correlation spectrum
\item[\texttt{REAL8 f0}] Start frequency for CC spectrum
\item[\texttt{REAL8 deltaF}] Frequency spacing for CC spectrum
\item[\texttt{CHAR *dmro1name}] Name of first data channel
\item[\texttt{CHAR *dmro2name}] Name of second data channel
\item[\texttt{CHAR *response1name}] Name of first response function
\item[\texttt{CHAR *response2name}] Name of second response function
\item[\texttt{UINT4 siteID1}] index of first detector in \texttt{lalCachedDetectors}
\item[\texttt{UINT4 siteID2}] index of second detector in \texttt{lalCachedDetectors}
\item[\texttt{CHAR *spectrum1name}] Name of first PSD
\item[\texttt{CHAR *spectrum2name}] Name of second PSD
\item[\texttt{REAL8 allegroLongitude}] Longitude of ALLEGRO
\item[\texttt{REAL8 allegroLatitude}] Latitude of ALLEGRO
\item[\texttt{REAL4 allegroAzimuth}] Azimuth of ALLEGRO
\end{description}

*************************************************** </lalLaTeX> */

typedef struct
tagStochasticInitParams
{
  UINT4                           numSegments;
  UINT4                           numPoints;
  UINT4                           numFrequencies;
  REAL8                           f0;
  REAL8                           deltaF;
  UINT4                           outputNumFreq;
  REAL8                           outputF0;
  REAL8                           outputDeltaF;
  CHAR                           *dmro1name;
  CHAR                           *dmro2name;
  CHAR                           *response1name;
  CHAR                           *response2name;
  BOOLEAN                         epochsMatch;
  UINT4                           siteID1;
  UINT4                           siteID2;
  CHAR                           *spectrum1name;
  CHAR                           *spectrum2name;
  REAL8				  allegroLongitude;
  REAL8				  allegroLatitude;
  REAL4				  allegroAzimuth;
}
StochasticInitParams;

/******************************* <lalLaTeX file="StochasticBarDataH">

\subsubsection*{\texttt{struct StochasticDataSegment}}
\index{\texttt{StochasticDataSegment}}

\noindent
The fields are:
\begin{description}
\item[\texttt{COMPLEX8TimeSeries *data1}]
\item[\texttt{COMPLEX8TimeSeries *data2}]
\end{description}

*************************************************** </lalLaTeX> */

typedef struct
tagStochasticDataSegment
{
    COMPLEX8TimeSeries*                data1;
    COMPLEX8TimeSeries*                data2;
}
StochasticDataSegment;

/******************************* <lalLaTeX file="StochasticBarDataH">

\subsubsection*{\texttt{struct StochasticDataSegmentVector}}
\index{\texttt{StochasticDataSegmentVector}}

\noindent
The fields are:
\begin{description}
\item[\texttt{UINT4 length}]
\item[\texttt{StochasticDataSegment *data}]
\end{description}

*************************************************** </lalLaTeX> */

typedef struct
tagStochasticDataSegmentVector
{
    UINT4                           length;
    StochasticDataSegment*          data;
}
StochasticDataSegmentVector;


void
LALCreateStochasticDataSegmentVector (
    LALStatus*                      status,
    StochasticDataSegmentVector**   vector,
    StochasticInitParams*           params
);


void
LALDestroyStochasticDataSegmentVector (
    LALStatus*                      status,
    StochasticDataSegmentVector**   vector
);


#ifdef  __cplusplus
}
#endif /* C++ protection. */

#endif /* _STOCHASTICDATA_H  */

/******************************* <lalLaTeX file="StochasticBarDataH">
  
\vfill{\footnotesize\input{StochasticBarDataHV}}

\newpage\input{StochasticBarDataC}

*************************************************** </lalLaTeX> */
