/************************** <lalVerbatim file="StochasticFinalizeSearchCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticFinalizeSearch.c,v 1.6 2001/08/16 00:29:42 whelan Exp $
**************************************************** </lalVerbatim> */

/*************************** <lalLaTeX file="StochasticFinalizeSearchC">
\subsection{Module \texttt{StochasticFinalizeSearch.c}}
\label{s:StochasticFinalizeSearch.c}

The LAL-wrapper interface function for ...

\subsection*{Description}

\subsection*{Algorithms}
  
\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{StochasticFinalizeSearchCV}}

*************************************************** </lalLaTeX> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/TimeFreqFFT.h>
#include "StochasticData.h"
#include "StochasticFinalizeSearch.h"

NRCSID (STOCHASTICFINALIZESEARCHC, "$Id: StochasticFinalizeSearch.c,v 1.6 2001/08/16 00:29:42 whelan Exp $" );

void
LALFinalizeSearch(
    LALStatus*          status,
    void**              searchParams
)
{
    StochasticSearchParams *params;

    
    INITSTATUS (status, "LALFinalizeSearch", STOCHASTICFINALIZESEARCHC);
    ATTATCHSTATUSPTR (status);

    
    /* 
     *
     * check and cast searchParams 
     *
     */
    
    /* :TODO: maybe an ASSERT is sufficient here?? */
    if (!(*searchParams)) {
        ABORT (status, 
               STOCHASTICFINALIZESEARCHH_ENULL, 
               STOCHASTICFINALIZESEARCHH_MSGENULL);
    }
    
    params = (StochasticSearchParams *) *searchParams;


    /*
     *
     * clean up
     *
     */
    
    /* free the memory used for the StochasticDataSegmentVector */
    LALDestroyStochasticDataSegmentVector (status->statusPtr, 
                                           &(params->stochasticSegVec));
    CHECKSTATUSPTR (status);

    /* free optimal filter */
    TRY( LALCDestroyVector (status->statusPtr, &(params->optimalFilter->data)),
	 status );

    LALFree (params->optimalFilter);

    /* free fft plan */
    TRY( LALDestroyRealFFTPlan (status->statusPtr, &(params->fftPlan)),
	 status );
    
    /* free string fields in StochasticInitParams */

    LALFree (params->initParams->dmro1name);
    params->initParams->dmro1name = NULL;
    LALFree (params->initParams->dmro2name);
    params->initParams->dmro2name = NULL;
    LALFree (params->initParams->response1name);
    params->initParams->response1name = NULL;
    LALFree (params->initParams->response2name);
    params->initParams->response2name = NULL;
    LALFree (params->initParams->spectrum1name);
    params->initParams->spectrum1name = NULL;
    LALFree (params->initParams->spectrum2name);
    params->initParams->spectrum2name = NULL;

    /* free StochasticInitParams */
    LALFree (params->initParams);
    params->initParams = NULL; 

    /* free StochasticSearchParams */
    LALFree (*searchParams);
    *searchParams = NULL;

    
    /* normal exit */
    DETATCHSTATUSPTR (status);
    RETURN (status);
}

